/************************************************************
	FILE:		lib_com.c
	CONTAINS:	simple commands
************************************************************/

#include "tcl.h"

#include "lib_defs.h"
#include "lib_globals.h"
#include "lib_com.h"

/************************************************************
	PROC:	quit(argc,argv)
	DOES:	lists dynamics equations available for use
************************************************************/

int quit(ClientData clientData,
		   Tcl_Interp *interp,
		   int argc, char **argv)
{
	exit(0);
}

/************************************************************
	PROC:	help
	DOES:	prints appropriate section from help file
************************************************************/

int help(ClientData clientData,
		   Tcl_Interp *interp,
		   int argc, char **argv)
{
	FILE *help_fp;			/* pointer to help file */
	STR	line;				/* line of help file */
	BOOL print=FALSE;		/* printing flag */
	BOOL found=FALSE;		/* found any matches? */
	BOOL mismatch=FALSE;	/* mismatch in comparison */
	int ch,n;				/* counters */
	int len;				/* line length */
	int wordlen;			/* length of search word */
	STR topic;              /* help topic */

	strcpy(topic,argc==1?"help":argv[1]);
	help_fp = fopen(HElpfile,"r");
	if (!help_fp) {
		fprintf(stderr,"help file not found\n");
		fprintf(stderr,"Expecting it to be: %s\n",HElpfile);
		return TCL_OK;
	}
	wordlen = strlen(topic);
	while (!feof(help_fp)) {						/* for each line */
		fgets(line,STRLEN,help_fp);
		if (line[0] == ':') {						/* a topic line? */
			print = FALSE;							/* turn off printing */
			len = strlen(line);
			ch = 0;
			while (1) {
				while (ch<len && line[ch]!=':')		/* look for ':' */
					ch++;
				if (ch>=len) break;					/* no more entries ? */
				ch++;
				mismatch = FALSE;
				for (n=0; n<wordlen; n++)			/* check for match */
					if (topic[n] != line[ch+n]) {
						mismatch = TRUE;
						ch += n;
						break;
					}
				if (!mismatch) {					/* found a match */
					print = found = TRUE;
				}
			}
		} else if (print)
			printf("%s",line);
	}
	if (!found) {
		printf("Help for %s not found.\n",topic);
		printf("Use 'help sum' for a summary of commands\n");
	}
	return TCL_OK;
}

/************************************************************
	PROC:	pipe_in
	DOES:	used to source input from files
************************************************************/

int pipe_in(ClientData clientData,
		   Tcl_Interp *interp,
		   int argc, char **argv)
{
	if (argc<2) {
       cerr(argc,argv,"Usage: %s <file>");
	   return TCL_OK;
	 }
	Tcl_EvalFile(interp,argv[1]);	/* read in file & interpret */
	return TCL_OK;
}

/************************************************************
	PROC:	cerr
	DOES:	handles command errors
************************************************************/

cerr(int argc, char **argv, char *errmsg)
{
	int n;

	fprintf(stderr,"ERROR in line %d: %s\n",Linenum,errmsg);
	fprintf(stderr,"===> ");
	for (n=0; n<argc; n++) fprintf(stderr,"%s ",argv[n]);
	fprintf(stderr,"\n");
}

