/************************************************************
	FILE: lib_main.C

************************************************************/

#define MAIN
#include <stdio.h>
#include <stdlib.h>
#include <tcl.h>
#include <tk.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include "glaux.h"

#include "lib_defs.h"
#include "lib_globals.h"
#include "lib_main.h"
#include "lib_math.h"
#include "lib_draw.h"
#include "lib_user.h"
#include "lib_image.h"

Tcl_Interp *Interp = NULL;        // begin with no interpreter

///////////////////////////////////////////////////////////
//    PROC:  main()
//    DOES:  does Tcl/Tk setup,
//           hands over control to a TK Mainloop,
//           which hands control over to main3() after
//           a timer interrupt
///////////////////////////////////////////////////////////

int main(int argc, char **argv)
{
    Tk_Main(argc, argv, Tcl_AppInit);
    return 0;			// never makes it here --  prevents compiler warning
}


///////////////////////////////////////////////////////////
//    PROC: Tcl_AppInit()
//    DOES: application-specific initialization
///////////////////////////////////////////////////////////

int Tcl_AppInit(Tcl_Interp *interp)
{
    Tk_Window main;

	Interp = interp;
    if (Tcl_Init(interp) == TCL_ERROR) return TCL_ERROR;
    if (Tk_Init(interp) == TCL_ERROR) return TCL_ERROR;
	Tcl_CreateCommand(interp, "redisplay", redisplay, (ClientData) NULL,
					  (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand(interp, "toggle_view", toggle_view, (ClientData) NULL,
					  (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand(interp, "help", help, (ClientData) NULL,
					  (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand(interp, "<", pipe_in, (ClientData) NULL,
					  (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand(interp, "quit", quit, (ClientData) NULL,
					  (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand(interp, "texload", texload, (ClientData) NULL,
					  (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand(interp, "texhead", texhead, (ClientData) NULL,
					  (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand(interp, "viewmat", viewmat, (ClientData) NULL,
					  (Tcl_CmdDeleteProc *) NULL);
	Tk_CreateTimerHandler(10, mytimer, (ClientData) NULL);
	Tcl_LinkVar(interp, "drawmode", (char *) &DRaw_mode, TCL_LINK_INT);
	Tcl_LinkVar(interp, "dbuffer", (char *) &DBuffer, TCL_LINK_INT);
//	tcl_RcFileName = "ui";
    Tcl_SetVar(interp, "tcl_rcFileName", "ui", TCL_GLOBAL_ONLY);
    return TCL_OK;
}

///////////////////////////////////////////////////////////
//    PROC: mytimer()
//    DOES: called upon a timer event;
//          really just an excuse to grab control of the event loop
///////////////////////////////////////////////////////////

void mytimer(ClientData clientData)
{
  lib_init();
  user_init();
  user_loop();
}

///////////////////////////////////////////////////////////
//    PROC: check_events()
//    DOES: processes both TK events and OpenGL events
///////////////////////////////////////////////////////////

void check_events()
{
  extern int GL_winopen;
  extern void my_qtkExec(void);

	while(Tk_DoOneEvent(TK_DONT_WAIT) != 0) {};
	if (GL_winopen) {         /* handle events in OpenGL window, if any */
		my_qtkExec();
	}
}

///////////////////////////////////////////////////////////
//	PROC:	start_event_loop(void)
//	DOES:	initializes globals and then takes over event processing
///////////////////////////////////////////////////////////

void start_event_loop(void)
{
  extern void event_loop(void);

  lib_init();			// initialize interface globals
  user_init();          // initialize user globals
  user_loop();          // hand over control to user
}

///////////////////////////////////////////////////////////
//	PROC:	init
//	DOES:	initializes globals
///////////////////////////////////////////////////////////

void lib_init(void)
{
  int n;
  float tf;

  tf = 2.0*M_PI/CIRC_PTS;
  for (n=0; n<CIRC_PTS; n++) {
	Circ[n][0] = cos(n*tf);
	Circ[n][1] = sin(n*tf);
  }
  DBuffer = TRUE;
  DEbug = FALSE;
  DRaw_mode = TEXTURE;
  strcpy(HElpfile,"help");
  Image = (IMAGE *) NULL;
  IMode = FALSE;
  MAtprint = FALSE;
  Mode = TRANSLATE_MODE;
  TExtures = FALSE;
  OpenGL_setup();
  idenmat(Vmat);
  Vmat[2][3] = -5;
  for (n=0; n<4; n++) {
    tm_copy(Vmat_mem[n], Vmat);
  }
//  load_vmat_mem();            // load most recent values from file
}








