///////////////////////////////
//
// Maciej Kalisiak
// a2504kal@cdf
// 009776021
//
// tabstop=4 !!!!!!!!!!!!!!!
//
// :vim:ts=4:sw=4:ai:
//////////////////////////////
//	FILE: terrain.C
//	DOES: procedures for the terrain class
//////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <iostream.h>
#include <fstream.h>
#include "tcl.h"
#include "glslib_defs.h"
#include "terrain.h"
#include "GL/gl.h"
#include "user.h"

#include "glslib_math.h"

TERRAIN Terrain;

void reset_colcache();				// resets the colour cache
void set_colour(float r, float g, float b);	// sets a drawing colour
// (this is just ripped out
// of question 4)

bool fSolidDraw=true;				// set to true to, and use
// ~a2504kal/pub/user.C
// to view the land drawn
// solid, and coloured based
// on altitude
									

/////////////////////////////////////////
//	PROC:	TERRAIN::TERRAIN()
//	DOES:	constructor for TERRAIN class
/////////////////////////////////////////

TERRAIN::TERRAIN()
{
  Z = (float *) 0;
  nlevels = 0;
  N = 0;
  N2 = 0;
  create(NLevels);
}


/////////////////////////////////////////
//	PROC:	TERRAIN::create(int)
//	DOES:	generates new terrain data, given #levels of detail
/////////////////////////////////////////

// Maciej Kalisiak
// a2504kal@cdf
// 009776021

void TERRAIN::create(int lev)
{
  nlevels = lev;           // save number of levels
  printf("New terrain, %d levels\n", nlevels);
  N = 1 << nlevels;
  N2 = N/2;
  if (Z) {            // delete old terrain data
    delete [] Z;
  }
  sf = 1.0/N;
  Dsize = (N+1)*(N2+1);
  Z = new float [Dsize];   // allocate space for new data
  for (int n=0; n<Dsize; n++)  // initialize to zero
    Z[n] = 0.0;           
  
  // use midpoint perturbation method to generate fractal shape
  // Here are some useful variables
  printf("AMp=%6.3f\n",AMp);      // amplitude factor
  // array is of size [0..N], [0..N] where N=2^nlevels, and
  //  only the bottom-right half elements are filled
  printf("N=%d\n",N);

  // your code goes here!



  printf("To view the landscape in SOLID mode, with height colouring,\n"
	 "change the boolean fSolidDraw to 'true' in terrain.C, and also\n"
	 "use 'user.C' found in ~a2504kal/pub\n");

  // set starting heights for the three vertices of array
  setZ( 0, 0, N*rand1()*AMp );
  setZ( N, 0, N*rand1()*AMp );
  setZ( N, N, N*rand1()*AMp );

  // set all other vertices of the array
  for( int skip=N; skip>1; skip/=2 ) {
    for( int y=0; y<N; y+=skip )
      for( int x=y; x<N; x+=skip ) {
	setZ( x+skip/2, y, 
	      ( getZ(x,y) + getZ(x+skip,y) )/2 + skip*rand1()*AMp );
	setZ( x+skip/2, y+skip/2, 
	      ( getZ(x,y) + getZ(x+skip,y+skip) )/2 + skip*rand1()*AMp );
	setZ( x+skip, y+skip/2, 
	      ( getZ(x+skip,y) + getZ(x+skip,y+skip) )/2 + skip*rand1()*AMp );
      }
  }

	
//   int seaLevel = 0;			// arbitrarily set

//   for( int y=0; y<=N; y++ )
//     for( int x=y; x<=N; x++ )
//       if( getZ(x,y) < seaLevel )
// 	setZ(x,y,seaLevel);

  dump_obj( "landscape.obj" );
}

/////////////////////////////////////////
//	PROC:	TERRAIN::draw_wire(int)
//	DOES:	draws wire-frame version of terrain
/////////////////////////////////////////

void TERRAIN::draw_wire(int lev)
{
  int step = 1 << (nlevels - lev);
  //  printf("Drawing wire-frame terrain, clevel=%d\n", lev);
  glColor3f(0,0,0);
  if (!Dsize) return;            // no terrain data yet?
  for (int y=0; y<N; y+=step) {
    glBegin(GL_LINE_STRIP);
    for (int x=y; x<N; x+=step) {
      vertex(x, y);
      vertex(x+step, y+step);
    }
    vertex(N,y);
    glEnd();

    glBegin(GL_LINE_STRIP);
    for (int x=y; x<=N; x+=step) {
      vertex(x,y);
    }
    glEnd();
  }
}


/////////////////////////////////////////
//	PROC:	TERRAIN::vertex(int x, int y)
//	DOES:	calculates 3D coordinates, given 
//          x,y position in terrain
/////////////////////////////////////////

void TERRAIN::vertex(int x, int y)
{
  const float COS60 = 0.866;
  FVEC v;
  static FVEC v1, v2;
  v[0] = (x - 0.5*y)*sf - 0.5;
  v[1] = y*COS60*sf - 0.43;
  v[2] = getZ(x,y)*sf;

  if( fSolidDraw ) {
    FVEC u,w,norm;
    vecsub( u, v2, v1 );
    vecsub( w, v, v1 );
    xprodnorm( norm, u, w );

    if( norm[2] < 0 ) {
      norm[0] *=-1;
      norm[1] *=-1;
      norm[2] *=-1;
    }

    glNormal3fv(norm);

    //  if( v[2]/sf > ((0.15+rand1()*0.02)*N*AMp00.0) )
    if( v[2]/sf > ((0.8+rand1()*0.03)*N*AMp) )
      set_colour(1,1,1);
    else if( v[2]>0 )
      set_colour(0.2,0.6,0);
    else if( v[2]<=0 )
      set_colour(0,0,0.5);
    else
      set_colour(1,0,0);
  }

  glVertex3fv(v);

  v1 = v2;
  v2 = v;
}

/////////////////////////////////////////
//	PROC:	TERRAIN::draw_solid(int)
//	DOES:	draws solid-shaded version of terrain
/////////////////////////////////////////

void TERRAIN::draw_solid(int lev)
{
  reset_colcache();

  //  printf("Drawing solid-shaded terrain, clevel=%d\n", lev);
  int step = 1 << (nlevels - lev);

  if (!Dsize) return;            // no terrain data yet?
  for (int y=0; y<N; y+=step) {
    reset_colcache();
    glBegin(GL_TRIANGLE_STRIP);
    for (int x=y; x<N; x+=step) {
      vertex(x, y);
      vertex(x+step, y+step);
    }
    vertex(N,y);
    glEnd();
  }
}

/////////////////////////////////////////
//	PROC:	TERRAIN::setZ()
//	DOES:	sets a height value
/////////////////////////////////////////

void TERRAIN::setZ(int x, int y, float val)
{
  int i;
  if (y>N2) {
    x = N - x;
    y = N - y + 1;
  }
  i = y*(N+1) + x;
  assert(i>=0 && i<Dsize);
  Z[i] = val;
}

/////////////////////////////////////////
//	PROC:	TERRAIN::getZ()
//	DOES:	gets a height value
/////////////////////////////////////////

float TERRAIN::getZ(int x, int y)
{
  int i;
  if (y>N2) {
    x = N - x;
    y = N - y + 1;
  }
  i = y*(N+1) + x;
  //printf("i=%d, Dsize=%d\n",i,Dsize);
  assert(i>=0 && i<Dsize);
  return Z[i];
}

///////////////////////////////////////////////////////////
//    PROC: rand1()
//    DOES: returns a random number in [-1,1)
///////////////////////////////////////////////////////////

float rand1()
{
  return 2.0*rand()/RAND_MAX - 1.0;
}



//////////////////////////////
// colour cache
//////////////////////////////
float cacheR, cacheG, cacheB;

//////////////////////////////////////////////
// this routine resets the cached colour
//////////////////////////////////////////////

void reset_colcache() 
{
  cacheR = cacheG = cacheB = -1;
}

////////////////////////////////////////////////////////////
//    PROC: set_colour();
//    DOES: draws a teapot of the given colour -- don't change
//
//	THIS ROUTINE IS RIPPED FROM QUESTION 4
/////////////////////////////////////////////////////////////


void set_colour(float r, float g, float b)
{

  if( r==cacheR && g==cacheG && b==cacheB )
    return;

  cacheR=r;
  cacheG=g;
  cacheB=b;

  float ambient = 0.2;
  float diffuse = 0.7;
  float specular = 0.4;
  GLfloat mat[4];
  /**** set ambient lighting parameters ****/
  mat[0] = ambient*r;
  mat[1] = ambient*g;
  mat[2] = ambient*b;
  mat[3] = 1.0;
  glMaterialfv (GL_FRONT, GL_AMBIENT, mat);

  /**** set diffuse lighting parameters ******/
  mat[0] = diffuse*r;
  mat[1] = diffuse*g;
  mat[2] = diffuse*b;
  mat[3] = 1.0;
  glMaterialfv (GL_FRONT, GL_DIFFUSE, mat);

  /**** set specular lighting parameters *****/
  mat[0] = specular*r;
  mat[1] = specular*g;
  mat[2] = specular*b;
  mat[3] = 1.0;
  glMaterialfv (GL_FRONT, GL_SPECULAR, mat);
  glMaterialf (GL_FRONT, GL_SHININESS, 0.5);
}


int TERRAIN::getNum( int x, int y ) {
  int xoff = x-y;
  return y*(2*N2+1) + 1 - ( (y-1)*(y-1)+(y-1))/2 + xoff;
}

void TERRAIN::dump_obj( char* szFname ) {

  fstream fout( szFname, ios::out );
  fstream foutF( "faces", ios::out );
  if( !fout || !foutF )
    return;

  const float COS60 = 0.866;

  for (int y=0; y<=N; y++) {
    for (int x=y; x<=N; x++) {
      fout << "v " << x-0.5*y <<" "<< y*COS60 <<" "<<getZ(x,y)<<endl;
      if( y ) {
	if( x>y ) {
	  foutF << "f "<<getNum(x-1,y-1)
		<<" "<<getNum(x-1,y)
		<<" "<<getNum(x,y)
		<<endl;
	}
	
	foutF << "f "<<getNum(x-1,y-1)<<" "<<getNum(x,y)<<" "
	  <<getNum(x,y-1)<<endl;
      }
    }
  }
}

