/************************************************************
 *
 *  Maciej Kalisiak
 *  a2504kal@cdf
 *  009776021
 *
 *	FILE:		user.C
 *	CONTAINS:	user functions
************************************************************/

#include "tcl.h"
#include "glaux.h"

#include "glslib_defs.h"
#include "glslib_globals.h"
#include "glslib_draw.h"
#include "glslib_user.h"

#include "user.h"      // add your prototypes here
#include "terrain.h"

int NLevels = 3;    // max number of levels for terrain
int CLevel = 3;     // current level of detail for display
double AMp = 0.25;   // default amplitude

extern bool fSolidDraw;		// see terrain.C for explanation

//////////////////////////////////////////////////
//	PROC:	user_gl_init(void)
//	DOES:	Initializes and OpenGL event handling
//          Probably not too useful, but you never know...
//////////////////////////////////////////////////

void user_gl_init(void)
{
  void key_toggle_view();       // defined later in this file

      // when V-key is pressed in OpenGL window,
      // call key_toggle_view()
      //  auxKeyFunc(AUX_v, key_toggle_view);
	  //
	
	if( fSolidDraw ) {		// if drawing solid land, get some light here
		GLfloat ambient[] = { 0.2, 0.2, 0.2, 1.0 };
		GLfloat diffuse[] = { 1, 1, 1, 1.0 };
		GLfloat specular[] = { 1.0, 1.0, 1.0, 1.0 };
		GLfloat position[] = { 1, 1, 0, 0.0 };
		
	//    GLfloat lmodel_ambient[] = { 0.1, 0.1, 0.1, 1.0 };
	//    GLfloat local_view[] = { 0.0 };

		GLfloat mat[4];

		glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
		glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
		glLightfv(GL_LIGHT0, GL_SPECULAR, specular);
		glLightfv(GL_LIGHT0, GL_POSITION, position);
	//    glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
	//    glLightModelfv(GL_LIGHT_MODEL_LOCAL_VIEWER, local_view);

		glEnable(GL_LIGHTING);
		glEnable(GL_LIGHT0);
		glEnable(GL_AUTO_NORMAL);
		glEnable(GL_NORMALIZE);
		glEnable(GL_DEPTH_TEST);
		glDepthFunc(GL_LESS);
  }
}

//////////////////////////////////////////////////
//	PROC:	user_init(void)
//	DOES:	Initializes user-defined globals
//////////////////////////////////////////////////

void user_init(void)
{
	Tcl_CreateCommand(Interp, "generate", generate, (ClientData) NULL,
					  (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand(Interp, "test", test, (ClientData) NULL,
					  (Tcl_CmdDeleteProc *) NULL);
	Tcl_LinkVar(Interp, "nlevels", (char *) &NLevels, TCL_LINK_INT);
	Tcl_LinkVar(Interp, "clevel", (char *) &CLevel, TCL_LINK_INT);
	Tcl_LinkVar(Interp, "amp", (char *) &AMp, TCL_LINK_DOUBLE);
}

///////////////////////////////////////////////////
//	PROC:	user_loop(void)
//	DOES:	All the action happens here. 
//            Queries for both OpenGL and TK events
///////////////////////////////////////////////////

void user_loop(void)
{
  extern int Quit;
  extern void check_events(void);
  extern Tcl_Interp *Interp;
  STR string;

  while (1) {
	check_events();
  }
}

////////////////////////////////////////////////////
//	PROC:	user_draw(void)
//	DOES:	user's drawing code
////////////////////////////////////////////////////

void user_draw(void)
{
  if( fSolidDraw )
	Terrain.draw_solid(CLevel);
  else
	Terrain.draw_wire(CLevel);
}

/////////////////////////////////////////////////////
//    PROC: draw_world_texture()
//    DOES: draw everything (with textures)
/////////////////////////////////////////////////////

void draw_world_texture( void )
{
   /* X faces */
   glEnable(GL_TEXTURE_2D);
   glColor3f( 1.0, 0.0, 0.0 );
   glBegin( GL_POLYGON );
   glTexCoord2f(0.0,0.0);   glVertex3f( 1.0, 1.0, 1.0 );
   glTexCoord2f(1.0,0.0);   glVertex3f( 1.0, -1.0, 1.0 );
   glTexCoord2f(1.0,1.0);   glVertex3f( 1.0, -1.0, -1.0 );
   glTexCoord2f(0.0,1.0);   glVertex3f( 1.0, 1.0, -1.0 );
   glEnd();

   glBegin( GL_POLYGON );
   glTexCoord2f(0.0,0.0);   glVertex3f( -1.0, 1.0, 1.0 );
   glTexCoord2f(1.0,0.0);   glVertex3f( -1.0, 1.0, -1.0 );
   glTexCoord2f(1.0,1.0);   glVertex3f( -1.0, -1.0, -1.0 );
   glTexCoord2f(0.0,1.0);   glVertex3f( -1.0, -1.0, 1.0 );
   glEnd();

   /* Y faces */
   glDisable(GL_TEXTURE_2D);
   glColor3f( 0.0, 1.0, 0.0 );
   glBegin( GL_POLYGON );
   glVertex3f(  1.0, 1.0,  1.0 );
   glVertex3f(  1.0, 1.0, -1.0 );
   glVertex3f( -1.0, 1.0, -1.0 );
   glVertex3f( -1.0, 1.0,  1.0 );
   glEnd();

   glBegin( GL_POLYGON );
   glVertex3f(  1.0, -1.0,  1.0 );
   glVertex3f( -1.0, -1.0,  1.0 );
   glVertex3f( -1.0, -1.0, -1.0 );
   glVertex3f(  1.0, -1.0, -1.0 );
   glEnd();

   glEnable(GL_TEXTURE_2D);

   /* Z faces */
   glColor3f( 0.0, 0.0, 1.0 );
   glBegin( GL_POLYGON );
   glTexCoord2f(0.0,0.0);   glVertex3f(  1.0,  1.0,  1.0 );
   glTexCoord2f(0.5,0.0);   glVertex3f( -1.0,  1.0,  1.0 );
   glTexCoord2f(0.5,0.5);   glVertex3f( -1.0, -1.0,  1.0 );
   glTexCoord2f(0.0,0.5);   glVertex3f(  1.0, -1.0,  1.0 );
   glEnd();

   glBegin( GL_POLYGON );
   glTexCoord2f(0.0,0.0);   glVertex3f(  1.0, 1.0, -1.0 );
   glTexCoord2f(2.0,0.0);   glVertex3f(  1.0,-1.0, -1.0 );
   glTexCoord2f(2.0,2.0);   glVertex3f( -1.0,-1.0, -1.0 );
   glTexCoord2f(0.0,2.0);   glVertex3f( -1.0, 1.0, -1.0 );
   glEnd();
}

/////////////////////////////////////////////////////
//    PROC: draw_world_solid()
//    DOES: draw everything, solid-shading
/////////////////////////////////////////////////////

void draw_world_solid( void )
{
   /* X faces */
   glColor3f( 1.0, 0.0, 0.0 );
   glBegin( GL_POLYGON );
   glVertex3f( 1.0, 1.0, 1.0 );
   glVertex3f( 1.0, -1.0, 1.0 );
   glVertex3f( 1.0, -1.0, -1.0 );
   glVertex3f( 1.0, 1.0, -1.0 );
   glEnd();

   glBegin( GL_POLYGON );
   glVertex3f( -1.0, 1.0, 1.0 );
   glVertex3f( -1.0, 1.0, -1.0 );
   glVertex3f( -1.0, -1.0, -1.0 );
   glVertex3f( -1.0, -1.0, 1.0 );
   glEnd();

   /* Y faces */
   glColor3f( 0.0, 1.0, 0.0 );
   glBegin( GL_POLYGON );
   glVertex3f(  1.0, 1.0,  1.0 );
   glVertex3f(  1.0, 1.0, -1.0 );
   glVertex3f( -1.0, 1.0, -1.0 );
   glVertex3f( -1.0, 1.0,  1.0 );
   glEnd();

   glBegin( GL_POLYGON );
   glVertex3f(  1.0, -1.0,  1.0 );
   glVertex3f( -1.0, -1.0,  1.0 );
   glVertex3f( -1.0, -1.0, -1.0 );
   glVertex3f(  1.0, -1.0, -1.0 );
   glEnd();

   /* Z faces */
   glColor3f( 0.0, 0.0, 1.0 );
   glBegin( GL_POLYGON );
   glVertex3f(  1.0,  1.0,  1.0 );
   glVertex3f( -1.0,  1.0,  1.0 );
   glVertex3f( -1.0, -1.0,  1.0 );
   glVertex3f(  1.0, -1.0,  1.0 );
   glEnd();

   glBegin( GL_POLYGON );
   glVertex3f(  1.0, 1.0, -1.0 );
   glVertex3f(  1.0,-1.0, -1.0 );
   glVertex3f( -1.0,-1.0, -1.0 );
   glVertex3f( -1.0, 1.0, -1.0 );
   glEnd();
}

///////////////////////////////////////////////////
//    PROC: draw_world_wire()
//    DOES: draw everything, wireframe
///////////////////////////////////////////////////

void draw_world_wire( void )
{
   /* X faces */
   glColor3f( 1.0, 0.0, 0.0 );
   glBegin( GL_LINE_LOOP );
   glVertex3f( 1.0, 1.0, 1.0 );
   glVertex3f( 1.0, -1.0, 1.0 );
   glVertex3f( 1.0, -1.0, -1.0 );
   glVertex3f( 1.0, 1.0, -1.0 );
   glEnd();

   glBegin( GL_LINE_LOOP );
   glVertex3f( -1.0, 1.0, 1.0 );
   glVertex3f( -1.0, 1.0, -1.0 );
   glVertex3f( -1.0, -1.0, -1.0 );
   glVertex3f( -1.0, -1.0, 1.0 );
   glEnd();

   /* Y faces */
   glColor3f( 0.0, 1.0, 0.0 );
   glBegin( GL_LINE_LOOP );
   glVertex3f(  1.0, 1.0,  1.0 );
   glVertex3f(  1.0, 1.0, -1.0 );
   glVertex3f( -1.0, 1.0, -1.0 );
   glVertex3f( -1.0, 1.0,  1.0 );
   glEnd();

   glBegin( GL_LINE_LOOP );
   glVertex3f(  1.0, -1.0,  1.0 );
   glVertex3f( -1.0, -1.0,  1.0 );
   glVertex3f( -1.0, -1.0, -1.0 );
   glVertex3f(  1.0, -1.0, -1.0 );
   glEnd();

   /* Z faces */
   glColor3f( 0.0, 0.0, 1.0 );
   glBegin( GL_LINE_LOOP );
   glVertex3f(  1.0,  1.0,  1.0 );
   glVertex3f( -1.0,  1.0,  1.0 );
   glVertex3f( -1.0, -1.0,  1.0 );
   glVertex3f(  1.0, -1.0,  1.0 );
   glEnd();

   glBegin( GL_LINE_LOOP );
   glVertex3f(  1.0, 1.0, -1.0 );
   glVertex3f(  1.0,-1.0, -1.0 );
   glVertex3f( -1.0,-1.0, -1.0 );
   glVertex3f( -1.0, 1.0, -1.0 );
   glEnd();
}


/*********************************************************
    PROC: key_toggle_view()
    DOES: toggles between rotate & translate modes
**********************************************************/

void key_toggle_view()
{
  switch(Mode) {
    case TRANSLATE_MODE:
      Mode = ROTATE_MODE;
      break;
    case ROTATE_MODE:
      Mode = TRANSLATE_MODE;
      break;
	}
  display();
}

///////////////////////////////////////////////////////////
//    PROC: generate()
//    DOES: builds a fractal terrain by triangle subdivision
///////////////////////////////////////////////////////////

int generate(ClientData clientData,
		   Tcl_Interp *interp,
		   int argc, char **argv)
{
  Terrain.create(NLevels);
  return TCL_OK;
}


///////////////////////////////////////////////////////////
//    PROC: test()
//    DOES: example of how to bind a new tcl function
///////////////////////////////////////////////////////////

int test(ClientData clientData,
		   Tcl_Interp *interp,
		   int argc, char **argv)
{
  printf("test()\n");
  return TCL_OK;
}

